import { EditorView } from '@codemirror/view';
import { EditorStateConfig } from '@codemirror/state';

declare type MergeConfig = {
    /**
    Configuration for the first editor (the left one in a
    left-to-right context).
    */
    a: EditorStateConfig;
    /**
    Configuration for the second editor.
    */
    b: EditorStateConfig;
    /**
    Parent element to append the view to.
    */
    parent?: Element | DocumentFragment;
    /**
    An optional root. Only necessary if the view is mounted in a
    shadow root or a document other than the global `document`
    object.
    */
    root?: Document | ShadowRoot;
    /**
    Controls whether revert controls are shown between changed
    chunks.
    */
    revertControls?: "a-to-b" | "b-to-a";
    /**
    When given, this function is called to render the button to
    revert a chunk.
    */
    renderRevertControl?: () => HTMLElement;
    /**
    By default, the merge view will mark inserted and deleted text
    in changed chunks. Set this to false to turn that off.
    */
    highlightChanges?: boolean;
    /**
    Controls whether a gutter marker is shown next to changed lines.
    */
    gutter?: boolean;
    /**
    When given, long stretches of unchanged text are collapsed.
    `margin` gives the number of lines to leave visible after/before
    a change (default is 3), and `minSize` gives the minimum amount
    of collapsible lines that need to be present (defaults to 4).
    */
    collapseUnchanged?: {
        margin?: number;
        minSize?: number;
    };
};
/**
A merge view manages two editors side-by-side, highlighting the
difference between them and vertically aligning unchanged lines.
If you want one of the editors to be read-only, you have to
configure that in its extensions.

By default, views are not scrollable. Style them (`.cm-mergeView`)
with a height and `overflow: auto` to make them scrollable.
*/
declare class MergeView {
    /**
    The first editor.
    */
    a: EditorView;
    /**
    The second editor.
    */
    b: EditorView;
    /**
    The outer DOM element holding the view.
    */
    dom: HTMLElement;
    private editorDOM;
    private revertDOM;
    private revertToA;
    private renderRevert;
    private chunks;
    private measuring;
    /**
    Create a new merge view.
    */
    constructor(config: MergeConfig);
    private dispatch;
    private scheduleMeasure;
    private measure;
    private updateRevertButtons;
    private renderRevertButton;
    private revertClicked;
    /**
    Destroy this merge view.
    */
    destroy(): void;
}

/**
Diffs are represented as arrays of changed ranges.
*/
declare type Changes = readonly {
    fromA: number;
    toA: number;
    fromB: number;
    toB: number;
}[];
/**
Compute the difference between two strings.
*/
declare const diff: (a: string, b: string) => Changes;
/**
Compute the difference between the given strings, and clean up the
resulting diff for presentation to users by dropping short
unchanged ranges, and aligning changes to word boundaries when
appropriate.
*/
declare const presentableDiff: (a: string, b: string) => Changes;

export { Changes, MergeView, diff, presentableDiff };
